{
  (* No header *)
}

let BLANK = [' ' '\t' '\n']

let LINE = [^ '\n']* '\n'
let COMMENT = '#' LINE

let NUM = ['0'-'9']
let ALPHA =  ['a'-'z' 'A'-'Z' '-' '_' ]
let WORD = ALPHA (ALPHA | NUM)*
let NUMBER = '-'? ['0'-'9']+

let ANY = _

let LPAR = '('
let RPAR = ')'
let BEGIN = 'b' 'e' 'g' 'i' 'n'
let END = 'e' 'n' 'd'

let SKIP = 's' 'k' 'i' 'p'
let SEQ = ';'
let WHILE = 'w' 'h' 'i' 'l' 'e'
let DO = 'd' 'o'
let IF = 'i' 'f'
let THEN = 't' 'h' 'e' 'n'
let ELSE = 'e' 'l' 's' 'e'
let ASSIGN = ':' '='
let PRINT = 'p' 'r' 'i' 'n' 't'
let TRY = 't' 'r' 'y'
let WITH = 'w' 'i' 't' 'h'
let RAISE = 'r' 'a' 'i' 's' 'e'
let ARROW = '-' '>'

let PLUS = '+'
let TIMES = '*'
let EQUAL = '='
let LESS = '<'


let EOP = '.'

rule make_token = parse

  | BLANK		{make_token lexbuf}	(* Skip blanks and comments*)
  | COMMENT		{make_token lexbuf}

  | eof			{Token_EOP}	(* Give up on end of file *)

  | EOP                 {Token_EOP}

  | LPAR		{Token_LPar}
  | RPAR		{Token_RPar}
  | BEGIN		{Token_Begin}
  | END 		{Token_End}

  | SKIP                {Token_Skip}
  | SEQ                 {Token_Seq}
  | WHILE               {Token_While}
  | DO                  {Token_Do}
  | IF                  {Token_If}
  | THEN                {Token_Then}
  | ELSE                {Token_Else}
  | ASSIGN              {Token_Assign}
  | PRINT               {Token_Print}
  | TRY                 {Token_Try}
  | WITH                {Token_With}
  | ARROW               {Token_Arrow}
  | RAISE               {Token_Raise}

  | PLUS                {Token_Plus}
  | TIMES               {Token_Mult}
  | EQUAL               {Token_Equal}
  | LESS                {Token_Less}

  | NUMBER
    { 
      let s = (Lexing.lexeme lexbuf) 
      in Token_Num(int_of_string(s))
    }

  | WORD
      { 
	let s = (Lexing.lexeme lexbuf)
	in Token_Var(s)
      }

  | ANY (* Default case: just skip the character *)
      {
	let s = (Lexing.lexeme lexbuf) 
	in (print_string ("Bad char: " ^ s ^ ". Continuing...\n");
	    make_token lexbuf)
      }
      
{

let build_prog_from_string s =
  make_prog make_token (Lexing.from_string s)
;;

}
