/* File parser.mly */

%token Token_LPar Token_RPar Token_Begin Token_End

%token <int> Token_Num
%token <string> Token_Var

%token Token_Skip
%token Token_Assign
%token Token_Seq
%token Token_If Token_Then Token_Else
%token Token_While Token_Do
%token Token_Print
%token Token_Try Token_With Token_Arrow
%token Token_Raise

%token Token_Plus
%token Token_Mult
%token Token_Equal
%token Token_Less

%token Token_EOP

%start make_prog             /* Entry point */

%type  <prog> make_prog

%%

make_prog:
  | prog2 Token_EOP                       {$1}

prog2:
  | prog2 Token_Seq prog                  {Seq($1,$3)}
  | prog                                  {$1}
;

prog:
  | Token_LPar prog2 Token_RPar           {$2}
  | Token_Begin prog2 Token_End           {$2}

  | Token_Skip                           {Skip}

  | Token_Var Token_Assign expr          {Assign($1,$3)}

  | Token_While expr
      Token_Do prog2 Token_End
                                         {While($2,$4)}

  | Token_If expr
      Token_Then prog2 
      Token_Else prog2 Token_End
                                         {If($2,$4,$6)}

  | Token_Print expr                     {Print($2)}

  | Token_Try prog2
      Token_With Token_Var
      Token_Arrow prog2 Token_End
					 {Try($2,$4,$6)}

  | Token_Raise Token_Var		 {Raise($2)}

;

expr:
  | expr Token_Equal expr2                {Expr_Equal($1,$3)}
  | expr Token_Less expr2                 {Expr_Less($1,$3)}
  | expr2                                 {$1}

expr2:
  | expr2 Token_Plus expr3                 {Expr_Plus($1,$3)}
  | expr3                                  {$1}

expr3:
  | expr4                                  {$1}
  | expr3 Token_Mult expr4                 {Expr_Mult($1,$3)}

expr4:
  | Token_Num                             {Expr_Num($1)}
  | Token_Var		                  {Expr_Var($1)}
  | Token_LPar expr Token_RPar            {$2}
;

%%
